# coding=utf-8
import requests
import time
import threading


class ElectSingle:
    def __init__(self, JSESSIONID, TURNID, COURSEID: list, thread_number=10, max_try=None) -> None:
        '''
        COURSEID: list of the ElectTurnLessonTaskID of courses
        thread_number: thred number for each course
        max_try: max try times
        '''
        self.ELECTTURNID = TURNID
        self.COURSEID = COURSEID
        self.trymax = max_try
        self.stop = 0
        self.reqall = 0
        self.Nu = thread_number
        self.cookies = {
            'JSESSIONID': JSESSIONID
        }
        self.headers = {
            'Connection': 'keep-alive',
            'sec-ch-ua': '^\\^Chromium^\\^;v=^\\^92^\\^, ^\\^',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'X-CSRF-TOKEN': 'ab3849d5-4aa0-471c-86eb-adfc0916af73',
            'X-Requested-With': 'XMLHttpRequest',
            'sec-ch-ua-mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Origin': 'https://coursesel.umji.sjtu.edu.cn',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://coursesel.umji.sjtu.edu.cn/welcome.action',
            'Accept-Language': 'zh-CN,zh;q=0.9',
        }
        print(f"init: electturnid: {self.ELECTTURNID}, courses: {self.COURSEID}")
        print(f"using {thread_number} threads for each course")

    def sendreq(self, data):
        while(1):
            if(self.stop):
                # print(self.reqall)
                return
            ts = time.time()
            if self.trymax and self.reqall >= self.trymax:
                self.stop = 1
                # print(ts-self.start_time)
            realtime = int(round(ts * 1000))
            params = (
                ('_t', realtime),
            )
            response = requests.post('https://coursesel.umji.sjtu.edu.cn/tpm/doElect_ElectTurn.action',
                                     headers=self.headers, params=params, cookies=self.cookies, data=data)
            ret = response.content.decode(encoding='utf-8', errors='ignore')
            if(len(ret) > 1000):
                print("error")
                exit()
            if(str(ret).find("false") == -1):
                print("success, congrats")
                self.stop = 1
                return
            print(ret.strip())
            self.reqall += 1

    def run(self):
        s = []
        for i in self.COURSEID:
            data = f'jsonString=%7B%22electTurnId%22%3A%22{self.ELECTTURNID}%22%2C%22autoElect%22%3Atrue%2C%22lessonTasks%22%3A%5B%22{i}%22%5D%7D'
            for _ in range(self.Nu):
                s.append(threading.Thread(target=self.sendreq, args=(data,)))
        for i in range(len(s)):
            s[i].start()
        self.start_time = time.time()
        for i in range(len(s)):
            s[i].join()
        print("end")



from courseselcli.login import JaccountCLILogin


async def get_coursesel_jsid(enable_mask: bool):
    async with JaccountCLILogin("https://coursesel.umji.sjtu.edu.cn/") as cli:
        await cli.login(enable_mask)
        cookies = cli.get_cookies()
        return cookies["JSESSIONID"].value



from jaccount_cli import JaccountCLIAsyncIO
from getpass import getpass


class JaccountCLILogin(JaccountCLIAsyncIO):
    async def login(self, enable_mask=True):
        await self.init()
        captcha_ascii = self.captcha_generate_ascii()
        print("", captcha_ascii, "", sep="\n")
        captcha = input("Please enter the shown captcha: ")
        username = input("Please enter jaccount username: ")
        if enable_mask:
            password = getpass("Please enter password: ")
        else:
            password = input("Please enter password: ")
        await super().login(username, password, captcha)



#!/usr/bin/env python3
'''
@Author: King
@Date: 2022-12-23 23:46:21
@Email: linsy_king@sjtu.edu.cn
@Url: https://yydbxx.cn
'''

import typer
import json

app = typer.Typer(add_completion=False)


def get_jsession():
    import courseselcli.coursesel as coursesel
    import asyncio
    print("Getting your JSESSIONID...")
    try:
        res = asyncio.run(coursesel.get_coursesel_jsid(True))
    except Exception:
        print("Failed to get JSESSIONID, please try again.")
        return
    return res


@app.command()
def init(use_realtime: bool = typer.Option(
        False, "--realtime", "-r", help="Use realtime request instead of preview request.")):
    """
    Initialize course elector
    """
    js = get_jsession()
    from courseselcli.elector import JIEelector
    elector = JIEelector(js, use_realtime)
    elector.run()
    print("Done. Please use command 'elect' to start electing.")


@app.command()
def refresh():
    """
    Refresh JSESSIONID in current configuration file
    """
    import os
    if not os.path.exists('.coursesel'):
        print('Not found .coursesel file. Please run init first')
        return
    js = get_jsession()
    with open('.coursesel', 'r') as f:
        config = json.load(f)
    config['jsessionID'] = js
    with open('.coursesel', 'w') as f:
        json.dump(config, f, indent=4)
    print("Done. Please use command 'elect' to start electing.")


@app.command()
def elect(jsessionID: str = typer.Option(None, "--jsessionID", "-j", help="Your JSESSIONID"),
          electTurnID: str = typer.Option(None, "--electTurnID", "-e"),
          ElectTurnLessonTaskID: str = typer.Option(None, "--electTurnLessonTaskID", "-l",
                                                    help="List of all courses you want to select, separated by comma (,)."),
          thread_number: int = typer.Option(10, "--thread", "-x",
                                            help="Number of threads to use for each course."),
          max_try: int = typer.Option(None, "--max-try", "-m",
                                      help="Maximum number of requests to send for each course. If not set, will try forever.")
          ):
    """
    Elect courses
    """
    import os
    if os.path.exists('.coursesel'):
        # Use this file to initialize
        with open('.coursesel', 'r') as f:
            config = json.load(f)
        jsessionID = config['jsessionID']
        electTurnID = config['electTurnId']
        courses_eid = []
        for l in config['courses']:
            courses_eid.append(l['electTurnLessonTaskId'])
        ElectTurnLessonTaskID = ','.join(courses_eid)
    if jsessionID is None:
        print("Please specify JSESSIONID")
        return
    if electTurnID is None:
        print("Please specify electTurnID")
        return
    if ElectTurnLessonTaskID is None:
        print("Please specify ElectTurnLessonTaskID")
        return
    JSESSIONID = jsessionID
    from courseselcli.single import ElectSingle

    my_elector = ElectSingle(JSESSIONID, electTurnID, ElectTurnLessonTaskID.split(
        ','), thread_number, max_try)
    my_elector.run()


if __name__ == "__main__":
    app()



#!/usr/bin/env python3
'''
@Author: King
@Date: 2022-12-23 23:49:52
@Email: linsy_king@sjtu.edu.cn
@Url: https://yydbxx.cn
'''

import json

# Selector class


class JIEelector:
    def __init__(self, jsessionid: str, enable_istothetime : bool = False) -> None:
        '''
        Set headers and cookies by jsessionid
        '''
        self.cookies = {
            'JSESSIONID': jsessionid
        }

        self.headers = {
            'Connection': 'keep-alive',
            'sec-ch-ua': '^\\^Chromium^\\^;v=^\\^92^\\^, ^\\^',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'X-CSRF-TOKEN': 'ab3849d5-4aa0-471c-86eb-adfc0916af73',
            'X-Requested-With': 'XMLHttpRequest',
            'sec-ch-ua-mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Origin': 'https://coursesel.umji.sjtu.edu.cn',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://coursesel.umji.sjtu.edu.cn/welcome.action',
            'Accept-Language': 'zh-CN,zh;q=0.9',
        }
        if enable_istothetime:
            self.enable_istothetime = "true"
        else:
            self.enable_istothetime = "false"
            

    def get_elect_turns(self):
        '''
        Get elect turns
        '''
        import requests
        response = requests.get('https://coursesel.umji.sjtu.edu.cn/tpm/findStudentElectTurns_ElectTurn.action',
                               headers=self.headers, cookies=self.cookies)
        all_turns = json.loads(response.text)

        if len(all_turns) == 0:
            print('No elect turn found.')
            exit(0)
        turn = all_turns[0]
        print(f"ElectTurn: {turn['electTurnName']}, Mode: {turn['electModeName']}, Start Time: {turn['beginTime']}")
        return turn["electTurnId"]

    def get_all_courses(self, electurnid: str):
        '''
        Get all courses in a turn
        '''
        from prettytable import PrettyTable
        import requests

        response = json.loads(requests.get(f'https://coursesel.umji.sjtu.edu.cn/tpm/findLessonTasks_ElectTurn.action?jsonString=%7B%22isToTheTime%22%3A{self.enable_istothetime}%2C%22electTurnId%22%3A%22{electurnid}%22%2C%22loadCourseGroup%22%3Atrue%2C%22loadElectTurn%22%3Atrue%2C%22loadCourseType%22%3Atrue%2C%22loadCourseTypeCredit%22%3Atrue%2C%22loadElectTurnResult%22%3Atrue%2C%22loadStudentLessonTask%22%3Atrue%2C%22loadPrerequisiteCourse%22%3Atrue%2C%22lessonCalendarWeek%22%3Afalse%2C%22loadLessonCalendarConflict%22%3Afalse%2C%22loadTermCredit%22%3Atrue%2C%22loadLessonTask%22%3Atrue%2C%22loadDropApprove%22%3Atrue%2C%22loadElectApprove%22%3Atrue%7D', headers=self.headers, cookies=self.cookies).text)

        mdata = response['data']['lessonTasks']
        table = PrettyTable(['ID', 'Name', 'Teacher', 'Code', 'Registration Satus'])

        for id, lesson in enumerate(mdata):
            cname = lesson['courseName']
            lclasscode = lesson['lessonClassCode']
            maxnum = lesson['maxNum']
            detail = lesson['lessonClassName']
            hasstu = lesson['studiedStudentNum']
            if 'lessonTaskTeam' in lesson:
                teacher = lesson['lessonTaskTeam']
            else:
                teacher = '未知'
            table.add_row([id, f'{detail}/{cname}', teacher,
                          lclasscode, f"{hasstu}/{maxnum}"])
        print(table)
        selected = input('Please enter the course ID(s) you want to elect, separated by commas:').split(',')
        table.clear_rows()

        if len(selected) == 0:
            print('No course selected')
            return

        # Confirm

        print('You selected:')

        selected_courses = [mdata[int(i)] for i in selected]

        for id, lesson in enumerate(selected_courses):
            cname = lesson['courseName']
            lclasscode = lesson['lessonClassCode']
            maxnum = lesson['maxNum']
            detail = lesson['lessonClassName']
            hasstu = lesson['studiedStudentNum']
            if 'lessonTaskTeam' in lesson:
                teacher = lesson['lessonTaskTeam']
            else:
                teacher = 'Unknown'
            if lesson['electTurnId'] != electurnid:
                print('ElecturnID not corrected:', cname)
                return
            if 'electTurnLessonTaskId' not in lesson:
                print('This course is not available:', cname)
                return
            table.add_row([id, f'{detail}/{cname}', teacher,
                          lclasscode, f"{hasstu}/{maxnum}"])
        print(table)

        # Export

        obj = {
            "jsessionID": self.cookies['JSESSIONID'],
            "electTurnId": electurnid,
            "courses": selected_courses
        }

        with open('.coursesel', 'w') as f:
            json.dump(obj, f, ensure_ascii=False, indent=4)
        
        print('Saved to .coursesel.')

    def run(self):
        '''
        Init
        '''
        turn = self.get_elect_turns()

        self.get_all_courses(turn)
